#include "mpi.h"
#include<math.h>
#include<stdlib.h>
#include<stdio.h>

double numint(double lo, double hi, int n, double res);
double f(double x);

/* ============  Worker-Teil =======================*/
void worker(int ipfa,int n,int ndom){
   int ipwo, np, i, iw, ipw, nl, tag_fa, tag_wo, tag_cl;
   double pia, lo, hi, delt, width, t;
   MPI_Status stat;
   MPI_Comm_rank(MPI_COMM_WORLD,&ipwo);
   MPI_Comm_size(MPI_COMM_WORLD,&np);
   tag_fa = 0; tag_wo = 1; tag_cl = 2;
   t = MPI_Wtime();
   width = 1./((double) ndom);
   MPI_Send(&ipwo, 1, MPI_INT, ipfa, tag_wo, MPI_COMM_WORLD);   
   pia = 0.0;
   for (i = 1; i<= ndom+2;i++){
      MPI_Recv( &iw, 1, MPI_INT, ipfa, tag_fa, MPI_COMM_WORLD, &stat );
      if (iw > ndom) break;
      lo = 1. -iw * width;
      hi = lo + width;
      nl = n * (f(lo) - f(hi))+1;
      delt = (hi-lo)/nl;
      pia = numint(lo,delt,nl,pia);
      MPI_Send(&ipwo, 1, MPI_INT, ipfa, tag_wo, MPI_COMM_WORLD);
   }
   t = MPI_Wtime()-t;
   MPI_Send(&pia, 1, MPI_DOUBLE, ipfa, tag_cl, MPI_COMM_WORLD );
   printf(" process %4i : %4i  domains in %10.2e sec,%15.5e \n",ipwo,i-1,t,pia);
   return;
}

/* ============  Farmer-Teil (me=0) =======================*/
double farmer(int idle,int ndom){
   int ipfa, np, ierr, iw, ipw, tag_fa, tag_wo, tag_cl;
   double pial, pia = 0.0;
   MPI_Status stat;
   MPI_Comm_rank(MPI_COMM_WORLD,&ipfa);  
   MPI_Comm_size(MPI_COMM_WORLD,&np);
   tag_fa = 0; tag_wo = 1; tag_cl = 2;
   for (iw = 1; iw <= ndom; iw++){
      MPI_Recv(&ipw, 1, MPI_INT, MPI_ANY_SOURCE,tag_wo,MPI_COMM_WORLD, &stat);
      MPI_Send(&iw, 1, MPI_INT, ipw, tag_fa, MPI_COMM_WORLD);
   }
   int istop = ndom+1;
   for (ipw = 1; ipw < np; ipw++){
      if (ipw == ipfa && idle == 0) continue;
      MPI_Send(&istop, 1, MPI_INT, ipw, tag_fa, MPI_COMM_WORLD);
   }
   for (ipw = 1; ipw < np; ipw++){
      if (ipw == ipfa && idle == 0) continue; 
      MPI_Recv(&pial, 1, MPI_DOUBLE, ipw, tag_cl,MPI_COMM_WORLD, &stat);
      pia = pia+pial;
   }
   return pia;
}
